/*

*/
import scene.*;
import scene.models.*;
import pipeline.*;
import framebuffer.*;

import java.io.File;
import java.awt.Color;

/**
   This program creates an animation by repeatedly saving the framebuffer's image to a file.

   You can see the animation by quickly sequencing through the frame images.
*/
public class OfflineModels
{
   public static void main(String[] args)
   {
      // Create several Model objects.
      Model m1 = new ObjSimpleModel(new File("assets/apple.obj"));
      Model m2 = new ObjSimpleModel(new File("assets/cow.obj"));
      Model m3 = new ObjSimpleModel(new File("assets/cessna.obj"));
      Model m4 = new ObjSimpleModel(new File("assets/galleon.obj"));
      Model m5 = new ObjSimpleModel(new File("assets/teapot.obj"));
      Model m6 = new ObjSimpleModel(new File("assets/small_rhombicosidodecahedron.obj"));
      Model m7 = new ObjSimpleModel(new File("assets/great_rhombicosidodecahedron.obj"));


      // Create the Scene object that we shall render
      Scene scene = new Scene();

      // Add Models to the Scene.
      scene.addModel(m1, m3, m4, m2);//apple, cessna, galleon, cow

      // Push the models away from where the camera is.
      for (Model m : scene.modelList)
         for (LineSegment ls : m.lineSegmentList)
         {
            ls.v[0].z -= 3;
            ls.v[1].z -= 3;
         }

      // Give the models random colors.
      for (Model m : scene.modelList)
      {
         m.setColorRandom();
      }

      // log interesting information to standard output
      //System.out.println( scene.toString() );

      // Create a framebuffer to render our scene into.
      int vp_width  = 512;
      int vp_height = 512;
      FrameBuffer fb = new FrameBuffer(vp_width, vp_height);
      // Give the framebuffer a nice background color.
      fb.clearFB(Color.black);

      // Render our scene into the frame buffer.
      Pipeline.render(scene, fb);
      // Save the resulting image in a file.
      fb.dumpFB2File( String.format("PPM_Frame%03d.ppm", 0) );

      for (int i = 1; i <= 100; i++)
      {
         // update the scene
         for (LineSegment ls : scene.modelList.get(0).lineSegmentList)
         {
            ls.v[0].x += 0.1;  // move right
            ls.v[1].x += 0.1;
            ls.v[0].y += 0.1;  // move up
            ls.v[1].y += 0.1;
            ls.v[0].z -= 0.1;  // move back
            ls.v[1].z -= 0.1;
         }
         for (LineSegment ls : scene.modelList.get(1).lineSegmentList)
         {
            ls.v[0].x -= 0.1;  // move left
            ls.v[1].x -= 0.1;
            ls.v[0].y -= 0.1;  // move down
            ls.v[1].y -= 0.1;
            ls.v[0].z -= 0.1;  // move back
            ls.v[1].z -= 0.1;
         }
         for (LineSegment ls : scene.modelList.get(2).lineSegmentList)
         {
            ls.v[0].x += 0.1; // move right
            ls.v[1].x += 0.1;
         }
         // Render again.
         fb.clearFB(Color.black);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_Frame%03d.ppm", i));
      }
   }
}
